@@ -1,3 +1,9 @@
+//! This library provides simple features for handling user's input and display for terminal applications.
+//! 
+//! Besides the user input and display, this library also provides some tools to build standalone "screens" that can be used as simply as printing it.
+//!
+//! It uses [Termion](https://crates.io/crates/termion) as main tool for handling the screen and inputs. You don't have to worry about initalizing anything because the lib will handle this for you.
+
 pub extern crate termion;
 
 mod utils;
@@ -14,22 +20,17 @@ use std::io::Write;
 use termion::input::{TermRead, MouseTerminal};
 
 /// Console Engine Framework
-pub struct ConsoleEngine {
-    input: termion::input::Events<termion::AsyncReader>,
-    output: MouseTerminal<termion::raw::RawTerminal<Stdout>>,
-    time_limit: u128,
-    /// The current frame count, publicly accessible
-    pub frame_count: usize,
-    width: u32,
-    height: u32,
-    screen: Screen,
-    screen_last_frame: Screen,
-    instant: std::time::Instant,
-    keys_pressed: Vec<Event>,
-    keys_held: Vec<Event>,
-    keys_released: Vec<Event>,
-}
-/// # Basic Usage :
+/// 
+/// # Features
+/// 
+/// *note : each link will redirect you to a bunch of functions*
+/// 
+/// - Build custom terminal display using [shapes](#method.line) or [text](#method.print)
+/// - Terminal handling with a [target frame per seconds](#method.init)
+/// - [Keyboard](#method.is_key_pressed) and [mouse](#method.get_mouse_press) support
+/// - [Terminal resizing](#method.check_resize) support
+/// 
+/// # Basic Usage:
 /// 
 /// ```
 /// use console_engine::pixel;
@@ -59,6 +60,25 @@ pub struct ConsoleEngine {
 ///     }
 /// }
 /// ```
+/// 
+/// # 
+/// 
+pub struct ConsoleEngine {
+    input: termion::input::Events<termion::AsyncReader>,
+    output: MouseTerminal<termion::raw::RawTerminal<Stdout>>,
+    time_limit: u128,
+    /// The current frame count, publicly accessible
+    pub frame_count: usize,
+    width: u32,
+    height: u32,
+    screen: Screen,
+    screen_last_frame: Screen,
+    instant: std::time::Instant,
+    keys_pressed: Vec<Event>,
+    keys_held: Vec<Event>,
+    keys_released: Vec<Event>,
+}
+
 impl ConsoleEngine {
     /// Initialize a screen of the provided width and height, and load the target FPS
     pub fn init(width: u32, height: u32, target_fps: u32) ->  ConsoleEngine {
@@ -153,6 +173,16 @@ impl ConsoleEngine {
     /// engine.print(0,0, String::from("Hello, world!"));
     /// engine.print(0, 4, format!("Score: {}", score));
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [lines-fps](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines-fps.rs)
+    /// - [screen-embed](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-embed.rs)
+    /// - [screen-simple](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-simple.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
+    /// - [snake](https://github.com/VincentFoulon80/console_engine/blob/master/examples/snake.rs)
     pub fn print(&mut self, x: i32, y: i32, string: String)
     {
         self.screen.print(x,y,string)
@@ -161,11 +191,16 @@ impl ConsoleEngine {
     /// prints a string at the specified coordinates with the specified foreground and background color  
     /// The string will automatically overlaps if it reach the right border
     /// 
-    /// usage
+    /// usage:
     /// ```
     /// // print "Hello, world" in blue on white background
     /// engine.print(0,0, String::from("Hello, world!"), color::Blue, color::White);
     /// ```
+    /// 
+    /// examples :
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [snake](https://github.com/VincentFoulon80/console_engine/blob/master/examples/snake.rs)
     pub fn print_fbg<C1: color::Color + Clone, C2: color::Color + Clone>(&mut self, x: i32, y: i32, string: String, fg: C1, bg: C2)
     {
         self.screen.print_fbg(x, y, string, fg, bg)
@@ -176,7 +211,7 @@ impl ConsoleEngine {
     /// 
     /// *see example* `screen-embed`
     /// 
-    /// usage :
+    /// usage:
     /// ```
     /// use console_engine::pixel;
     /// use console_engine::screen::Screen;
@@ -189,6 +224,9 @@ impl ConsoleEngine {
     /// // prints the square in the engine's screen at a specific location
     /// engine.print_screen(5,2, &my_square);
     /// ```
+    /// 
+    /// examples :
+    /// - [screen-embed](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-embed.rs)
     pub fn print_screen(&mut self, x : i32, y: i32, source: &Screen)
     {
         self.screen.print_screen(x, y, source)
@@ -199,12 +237,17 @@ impl ConsoleEngine {
     /// 
     /// Note : Your line can start or end out of bounds. These pixels won't be drawn 
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.line(0, 0, 9, 9, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [lines](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines.rs)
+    /// - [lines-fps](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines-fps.rs)
     pub fn line(&mut self, start_x: i32, start_y: i32, end_x: i32, end_y: i32, character: Pixel)
     {
         self.screen.line(start_x, start_y, end_x, end_y, character)
@@ -212,12 +255,19 @@ impl ConsoleEngine {
 
     /// Draws a rectangle of the provided character between two sets of coordinates  
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.rect(0, 0, 9, 9, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [screen-embed](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-embed.rs)
+    /// - [screen-simple](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-simple.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn rect(&mut self, start_x: i32, start_y: i32, end_x: i32, end_y: i32, character: Pixel)
     {
         self.screen.rect(start_x, start_y, end_x, end_y, character)
@@ -225,12 +275,15 @@ impl ConsoleEngine {
 
     /// Fill a rectangle of the provided character between two sets of coordinates  
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.fill_rect(0, 0, 9, 9, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn fill_rect(&mut self, start_x: i32, start_y: i32, end_x: i32, end_y: i32, character: Pixel)
     {
         self.screen.fill_rect(start_x, start_y, end_x, end_y, character)
@@ -239,12 +292,15 @@ impl ConsoleEngine {
     /// Draws a circle of the provided character at an x and y position with a radius
     /// see: [olcPixelGameEngine Repository](https://github.com/OneLoneCoder/olcPixelGameEngine)
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.circle(10, 10, 4, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn circle(&mut self, x: i32, y: i32, radius: u32, character: Pixel)
     {
         self.screen.circle(x, y, radius, character)
@@ -253,12 +309,17 @@ impl ConsoleEngine {
     /// Fill a circle of the provided character at an x and y position with a radius
     /// see: [olcPixelGameEngine Repository](https://github.com/OneLoneCoder/olcPixelGameEngine)
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.fill_circle(10, 10, 4, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [screen-simple](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-simple.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn fill_circle(&mut self, x: i32, y: i32, radius: u32, character: Pixel)
     {
         self.screen.fill_circle(x, y, radius, character)
@@ -266,12 +327,15 @@ impl ConsoleEngine {
 
     /// Draws a triangle of the provided character using three sets of coordinates
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.triangle(8,8, 4,6, 9,2, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn triangle(&mut self, x1: i32, y1: i32, x2: i32, y2: i32, x3: i32, y3: i32, character: Pixel)
     {
         self.screen.triangle(x1, y1, x2, y2, x3, y3, character)
@@ -280,12 +344,16 @@ impl ConsoleEngine {
     /// Fill a triangle of the provided character using three sets of coordinates
     /// see: [rustyPixelGameEngine Repository](https://github.com/mattbettcher/rustyPixelGameEngine)
     /// 
-    /// usage : 
+    /// usage: 
     /// ```
     /// use console_engine::pixel;
     /// // ...
     /// engine.fill_triangle(8,8, 4,6, 9,2, pixel::pxl('#'));
     /// ```
+    /// 
+    /// examples :
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
     pub fn fill_triangle(&mut self, x1: i32, y1: i32, x2: i32, y2: i32, x3: i32, y3: i32, character: Pixel)
     {
         self.screen.fill_triangle(x1, y1, x2, y2, x3, y3, character)
@@ -300,6 +368,12 @@ impl ConsoleEngine {
     /// // ...
     /// engine.set_pxl(3,8,pixel::pixel('o'));
     /// ```
+    /// 
+    /// examples :
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
+    /// - [snake](https://github.com/VincentFoulon80/console_engine/blob/master/examples/snake.rs)
     pub fn set_pxl(&mut self, x: i32, y: i32, character: Pixel)
     {
         self.screen.set_pxl(x, y, character)
@@ -313,6 +387,9 @@ impl ConsoleEngine {
     ///     engine.print(0,0,"Found a 'o'");
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - *no examples*
     pub fn get_pxl(&self, x: i32, y: i32) -> Result<Pixel, String> 
     {
         self.screen.get_pxl(x,y)
@@ -323,8 +400,11 @@ impl ConsoleEngine {
     /// 
     /// usage:
     /// ```
-    /// engine.resize()
+    /// engine.resize(40,10)
     /// ```
+    /// 
+    /// examples :
+    /// - *no examples*
     pub fn resize(&mut self, new_width: u32, new_height: u32)
     {
         self.screen.resize(new_width, new_height);
@@ -336,11 +416,31 @@ impl ConsoleEngine {
     /// Changes the screen instance used by the engine and updates internal informations
     /// 
     /// Useful if you want to manage multiple screens independently.
-    pub fn set_screen(&mut self, screen: Screen)
+    /// 
+    /// usage
+    /// ```
+    /// // create a new screen of 40x10 and draw some things on it
+    /// let mut scr = Screen::new(40,10)
+    /// scr.rect(0,0,39,9, pixel::pxl("#"));
+    /// // ...
+    /// 
+    /// // keep a backup of the old screen before replacing it
+    /// let old_scr = engine.get_screen();
+    /// // change the engine's current screen to the newly created one
+    /// engine.set_screen(&scr);
+    /// 
+    /// // ... later
+    /// // set back the old screen
+    /// engine.set_screen(&old_scr);
+    /// ```
+    /// 
+    /// examples :
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    pub fn set_screen(&mut self, screen: &Screen)
     {
         self.width = screen.get_width();
         self.height = screen.get_height();
-        self.screen = screen;
+        self.screen = screen.clone();
         self.screen_last_frame = Screen::from_vec(vec![], self.width, self.height);
     }
 
@@ -348,6 +448,16 @@ impl ConsoleEngine {
     /// 
     /// You can keep it into a variable to restore the screen later, via `set_screen`.
     /// You can then use the to_string method to write the screen in a file for example
+    /// 
+    /// see [set_screen](#method.set_screen) for a more complete example
+    /// 
+    /// usage :
+    /// ```
+    /// let scr = engine.get_screen(); 
+    /// ```
+    /// 
+    /// examples :
+    /// - *no examples*
     pub fn get_screen(&self) -> Screen
     {
         self.screen.clone()
@@ -361,6 +471,16 @@ impl ConsoleEngine {
     /// engine.print(0,0,String::from("Hello, world!")); // <- prints "Hello, world!" in 'screen' memory
     /// engine.draw(); // display 'screen' memory to the user's terminal
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [lines](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines.rs)
+    /// - [lines-fps](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines-fps.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
+    /// - [snake](https://github.com/VincentFoulon80/console_engine/blob/master/examples/snake.rs)
     pub fn draw(&mut self)
     {
         // we prepare an "output_screen" String variable to store in one-shot the screen we'll write.
@@ -427,6 +547,16 @@ impl ConsoleEngine {
     ///     // do your stuff
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [lines](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines.rs)
+    /// - [lines-fps](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines-fps.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
+    /// - [snake](https://github.com/VincentFoulon80/console_engine/blob/master/examples/snake.rs)
     pub fn wait_frame(&mut self) {
         let mut pressed: Vec<Event> = vec!();
 
@@ -466,6 +596,14 @@ impl ConsoleEngine {
     ///     // do your stuff
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [lines](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines.rs)
+    /// - [lines-fps](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines-fps.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn check_resize(&mut self)
     {
         if termion::terminal_size().unwrap() != (self.width as u16, self.height as u16) {
@@ -490,6 +628,16 @@ impl ConsoleEngine {
     ///     }
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [graph](https://github.com/VincentFoulon80/console_engine/blob/master/examples/graph.rs)
+    /// - [lines](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines.rs)
+    /// - [lines-fps](https://github.com/VincentFoulon80/console_engine/blob/master/examples/lines-fps.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
+    /// - [screen-swap](https://github.com/VincentFoulon80/console_engine/blob/master/examples/screen-swap.rs)
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
+    /// - [snake](https://github.com/VincentFoulon80/console_engine/blob/master/examples/snake.rs)
     pub fn is_key_pressed(&self, key: Key) -> bool
     {
         self.keys_pressed.contains(&Event::Key(key))
@@ -507,6 +655,9 @@ impl ConsoleEngine {
     ///     }
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [shapes](https://github.com/VincentFoulon80/console_engine/blob/master/examples/shapes.rs)
     pub fn is_key_held(&self, key: Key) -> bool
     {
         self.keys_held.contains(&Event::Key(key))
@@ -525,6 +676,9 @@ impl ConsoleEngine {
     ///     }
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - *no example*
     pub fn is_key_released(&self, key: Key) -> bool
     {
         self.keys_released.contains(&Event::Key(key))
@@ -532,7 +686,7 @@ impl ConsoleEngine {
 
     /// Give the mouse's terminal coordinates if the provided button has been pressed
     /// 
-    /// usage :
+    /// usage:
     /// ```
     /// // prints a 'P' where the mouse's left button has been pressed
     /// let mouse_pos = engine.get_mouse_press(termion::event::MouseButton::Left);
@@ -541,6 +695,10 @@ impl ConsoleEngine {
     ///    engine.set_pxl(mouse_pos.0 as i32, mouse_pos.1 as i32, pixel::pxl('P'));
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
     pub fn get_mouse_press(&self, button: termion::event::MouseButton) -> Option<(u32,u32)> 
     {
         for evt in self.keys_pressed.iter() {
@@ -563,7 +721,7 @@ impl ConsoleEngine {
 
     /// Give the mouse's terminal coordinates if a button is held on the mouse
     /// 
-    /// usage :
+    /// usage:
     /// ```
     /// // prints a 'H' where the mouse is currently held
     /// let mouse_pos = engine.get_mouse_held();
@@ -572,6 +730,10 @@ impl ConsoleEngine {
     ///     engine.set_pxl(mouse_pos.0 as i32, mouse_pos.1 as i32, pixel::pxl('H'));
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
     pub fn get_mouse_held(&self) -> Option<(u32,u32)> 
     {
         for evt in self.keys_pressed.iter() {
@@ -592,7 +754,7 @@ impl ConsoleEngine {
 
     /// Give the mouse's terminal coordinates if a button has been released on the mouse
     /// 
-    /// usage :
+    /// usage:
     /// ```
     /// // prints a 'R' where the mouse has been released
     /// let mouse_pos = engine.get_mouse_released();
@@ -601,6 +763,10 @@ impl ConsoleEngine {
     ///     engine.set_pxl(mouse_pos.0 as i32, mouse_pos.1 as i32, pixel::pxl('R'));
     /// }
     /// ```
+    /// 
+    /// examples :
+    /// - [drag-and-drop](https://github.com/VincentFoulon80/console_engine/blob/master/examples/drag-and-drop.rs)
+    /// - [mouse](https://github.com/VincentFoulon80/console_engine/blob/master/examples/mouse.rs)
     pub fn get_mouse_released(&self) -> Option<(u32,u32)> 
     {
         for evt in self.keys_pressed.iter() {
